"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = require("node:path");
const typescript_1 = __importDefault(require("typescript"));
const tools_1 = require("./tools");
class TsPathsPlugin {
    constructor() {
        this.name = 'ts-paths';
        this.setup = this.setup.bind(this);
        this.onResolve = this.onResolve.bind(this);
        this.onLoad = this.onLoad.bind(this);
    }
    async setup(build) {
        this.tsConfig = await (0, tools_1.getTsConfig)(build.initialOptions.absWorkingDir ?? process.cwd(), build.initialOptions.tsconfig ?? './tsconfig.json');
        if (!this.tsConfig?.options.paths) {
            throw new Error('tsconfig.json does not contain any path mappings');
        }
        this.program = typescript_1.default.createProgram({
            rootNames: this.tsConfig.fileNames,
            options: this.tsConfig.options,
            projectReferences: this.tsConfig.projectReferences,
            configFileParsingDiagnostics: this.tsConfig.errors,
        });
        this.searchRegExp = (0, tools_1.computeSearchRegExp)(this.tsConfig.options.paths);
        if (build.initialOptions.bundle) {
            build.onResolve({ filter: this.searchRegExp }, this.onResolve);
        }
        else {
            build.onLoad({ filter: /.*\.(m|c)?(j|t)sx?$/ }, this.onLoad);
        }
    }
    onResolve(args) {
        const sourceFile = this.program?.getSourceFile(args.importer);
        if (!sourceFile) {
            throw new Error(`Unable to find source file "${args.importer}"`);
        }
        const resolvedPath = this.resolveAlias(sourceFile, args.path, true);
        if (!resolvedPath) {
            return null;
        }
        return {
            path: resolvedPath,
        };
    }
    async onLoad({ path }) {
        if (!this.tsConfig) {
            throw new Error('Config should be defined at this point');
        }
        if (!this.tsConfig.options.paths) {
            throw new Error('paths mapping should be defined at this point');
        }
        if (!this.program) {
            throw new Error('Program should be defined at this point');
        }
        const sourceFile = this.program.getSourceFile(path);
        if (!sourceFile) {
            throw new Error(`Unable to find source file "${path}"`);
        }
        const contents = sourceFile.getFullText();
        if (!this.searchRegExp?.test(contents)) {
            return {
                contents,
                loader: 'ts',
            };
        }
        const paths = (0, tools_1.extractImportPaths)(sourceFile);
        const mapping = {};
        const errors = [];
        for (const importPath of paths) {
            if (!this.searchRegExp.test(importPath)) {
                continue;
            }
            const resolvedPath = this.resolveAlias(sourceFile, importPath);
            if (resolvedPath) {
                mapping[importPath] = resolvedPath;
            }
            else {
                errors.push({
                    text: `Unable to resolve "${importPath}" in "${path}"`,
                    location: {
                        file: path,
                        namespace: 'file',
                        line: 0,
                        column: 0,
                    },
                });
            }
        }
        return {
            contents: (0, tools_1.replacePath)(contents, mapping),
            loader: 'ts',
            errors,
        };
    }
    resolveAlias(sourceFile, imported, absolute = false) {
        if (!this.tsConfig?.options.paths) {
            throw new Error('tsconfig.json does not contain any path mappings');
        }
        if (!this.program) {
            throw new Error('Program should be defined at this point');
        }
        const { resolvedModule } = typescript_1.default.resolveModuleName(imported, sourceFile.fileName, this.tsConfig.options, typescript_1.default.sys);
        if (!resolvedModule) {
            throw new Error(`Unable to resolve "${imported}" from "${sourceFile.fileName}" with ${this.tsConfig.options.configFilePath}`);
        }
        const { resolvedFileName } = resolvedModule;
        if (absolute) {
            return resolvedFileName;
        }
        const relativePath = (0, node_path_1.relative)((0, node_path_1.dirname)(sourceFile.fileName), resolvedFileName).replace(/\.(j|t)sx?$/, '');
        return relativePath.startsWith('.') ? relativePath : `./${relativePath}`;
    }
}
exports.default = TsPathsPlugin;
//# sourceMappingURL=Plugin.js.map